/*
 *	@(#)MouseTranslate.java 1.21 98/11/24 16:13:17
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.sun.j3d.utils.behaviors.mouse;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

/**
 * MouseTranslate is a Java3D behavior object that lets users control the 
 * translation (X, Y) of an object via a mouse drag motion with the third
 * mouse button (alt-click on PC). See MouseRotate for similar usage info.
 */
 
public class MouseTranslate extends MouseBehavior {
    double x_factor = .02;
    double y_factor = .02;
    Vector3d translation = new Vector3d();

  private MouseBehaviorCallback callback = null;

  /**
   * Creates a mouse translate behavior given the transform group.
   * @param transformGroup The transformGroup to operate on.
   */
  public MouseTranslate(TransformGroup transformGroup) {
    super(transformGroup);
  }
   
  /**
   * Creates a default translate behavior.
   */
  public MouseTranslate(){
    super(0);
  }

  /**
   * Creates a translate behavior.
   * Note that this behavior still needs a transform
   * group to work on (use setTransformGroup(tg)) and
   * the transform group must add this behavior.
   * @param flags
   */
  public MouseTranslate(int flags) {
    super(flags);
  }

  public void initialize() {
    super.initialize();
    if ((flags & INVERT_INPUT) == INVERT_INPUT) {
       invert = true;
       x_factor *= -1;
       y_factor *= -1;
    }
  }

  /**
   * Return the x-axis movement multipler.
   **/

  public double getXFactor() {
    return x_factor;
  }
  
  /**
   * Return the y-axis movement multipler.
   **/

  public double getYFactor() {
    return y_factor;
  }
  
  /**
   * Set the x-axis amd y-axis movement multipler with factor.
   **/

  public void setFactor( double factor) {
    x_factor = y_factor = factor;
    
  }
  
  /**
   * Set the x-axis amd y-axis movement multipler with xFactor and yFactor
   * respectively.
   **/

  public void setFactor( double xFactor, double yFactor) {
    x_factor = xFactor;
    y_factor = yFactor;    
  }

  public void processStimulus (Enumeration criteria) {
    WakeupCriterion wakeup;
    AWTEvent[] event;
    int id;
    int dx, dy;
    
    while (criteria.hasMoreElements()) {
      wakeup = (WakeupCriterion) criteria.nextElement();
      
      if (wakeup instanceof WakeupOnAWTEvent) {
	event = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
	for (int i=0; i<event.length; i++) { 
	  processMouseEvent((MouseEvent) event[i]);

	  if (((buttonPress)&&((flags & MANUAL_WAKEUP) == 0)) ||
	      ((wakeUp)&&((flags & MANUAL_WAKEUP) != 0))){
	    id = event[i].getID();
	    if ((id == MouseEvent.MOUSE_DRAGGED) &&
		!((MouseEvent)event[i]).isAltDown() && 
		((MouseEvent)event[i]).isMetaDown()){
	      
	      
	      x = ((MouseEvent)event[i]).getX();
	      y = ((MouseEvent)event[i]).getY();
	      
	      dx = x - x_last;
	      dy = y - y_last;
	      
	      if ((!reset) && ((Math.abs(dy) < 50) && (Math.abs(dx) < 50))){
		//System.out.println("dx " + dx + " dy " + dy);
		transformGroup.getTransform(currXform);
		
		translation.x = dx*x_factor; 
		translation.y = -dy*y_factor;
		
		transformX.set(translation);
		
		if (invert) {
		  currXform.mul(currXform, transformX);
		} else {
		  currXform.mul(transformX, currXform);
		}
		
		transformGroup.setTransform(currXform);

		transformChanged( currXform );

		if (callback!=null)
		    callback.transformChanged( MouseBehaviorCallback.TRANSLATE,
					       currXform );

	      }
	      else {
		reset = false;
	      }
	      x_last = x;
	      y_last = y;
	    }
	    else if (id == MouseEvent.MOUSE_PRESSED) {
	      x_last = ((MouseEvent)event[i]).getX();
	      y_last = ((MouseEvent)event[i]).getY();
	    }
	  }
	}
      }
    }

    wakeupOn (mouseCriterion);
  }

  /**
    * Users can overload this method  which is called every time
    * the Behavior updates the transform
    *
    * Default implementation does nothing
    */
  public void transformChanged( Transform3D transform ) {
  }

  /**
    * The transformChanged method in the callback class will
    * be called every time the transform is updated
    */
  public void setupCallback( MouseBehaviorCallback callback ) {
      this.callback = callback;
  }
}

